@@ -1,8 +1,9 @@
 mod handler;
 mod https;
-mod middleware;
 mod service;
 
+pub mod middleware;
+
 use anyhow::Error;
 use hyper::service::{make_service_fn, service_fn};
 use std::net::{Ipv4Addr, SocketAddr};
